#ifndef PARTICLE_MANAGER_H_KEP093
#define PARTICLE_MANAGER_H_KEP093

#include "Particle.h"
#include <boost/timer.hpp>

/**
 * The ParticleManager uses a statically-sized array of particles.  Allocation
 * is done by scanning the list for a free particle.  While a Particle is
 * alive, its pointer will not change.  This avoids copying Particles around,
 * but increases the time to iterate.
 */
class ParticleManager {
public:
  typedef Particle* iterator;

public:
  explicit ParticleManager( int numParticles ) {
    assert( numParticles > 0 );
    parts = new Particle[ numParticles ];
    partsCurr = parts;
    partsEnd = parts + numParticles;
  }

  ~ParticleManager() {
    delete[] parts;
  }

  void addParticle( const Vector3D& pos, const Vector3D& velo, int color, float size, float life ) {
    findParticle()->reinit( pos, velo, color, size, life );
  }

  Particle* begin() {
    return parts;
  }

  Particle* end() {
    return partsEnd;
  }

private:
  /**
   * Finds a particle we can reinit.  Picks a dead particle, or else picks the
   * particle with the lowest remaining life.
   */
  Particle* findParticle() {
    //early exit if we are on a dead particle.
    if ( partsCurr->isDead() )
      return partsCurr;

    Particle* lowestParticle = partsCurr;
    Particle* particleStart = partsCurr;

    do {
      ++partsCurr;

      if ( partsCurr >= partsEnd ) //wrap around
        partsCurr = parts;
      if ( partsCurr == particleStart )
        return lowestParticle; //we reached the start, exit with lowest.

      if ( partsCurr->isDead() )
        return partsCurr;
      else if ( partsCurr->getLife() < lowestParticle->getLife() )
        lowestParticle = partsCurr;

    } while ( partsCurr != particleStart );

    //shouldn't get here.
    assert(false);
    return 0;
  }

private:
  Particle* parts;
  Particle* partsEnd;
  Particle* partsCurr;
};

#endif //PARTICLE_MANAGER_H_KEP093
